import {createEnhancedElement} from '@servicenow/library-enhanced-element';
import styles from './_now-highlighted-value.scss';
import '@servicenow/now-icon';
import truncationBehavior from '@servicenow/behavior-truncation';
import tooltipBehavior from '@servicenow/behavior-tooltip';
import '@servicenow/now-tooltip';

import {name, version} from '../package.json';
const registry = (window.___NOW_DESIGN_SYSTEM_PACKAGES___ =
	window.___NOW_DESIGN_SYSTEM_PACKAGES___ || {});
if (!registry[name]) {
	registry[name] = version;
}

const renderIcon = (icon, showIcon) => {
	if (icon || showIcon) {
		return (
			<now-icon
				icon={icon ? icon : 'presence-fill'}
				size="sm"
				className="now-highlighted-value-icon"
				aria-hidden="true"
			/>
		);
	}
};

const view = (state) => {
	const {label = '', status, color, variant, icon, showIcon} = state.properties;
	const statusOrColor = status || color;
	return (
		<span
			class={{
				'now-highlighted-value': true,
				['-' + statusOrColor]: statusOrColor,
				['-' + variant]: variant
			}}>
			{renderIcon(icon, showIcon)}
			<span className="now-highlighted-value-label">
				<span className="will-truncate" data-truncation>
					{label}
				</span>
			</span>
		</span>
	);
};

/**
 * Highlighted values communicate the value of an item
 * by promoting visibility and readability.
 *
 * Use highlighted values when you want to convey the status in a certain area,
 * such as a card, list, or widget. A highlighted value may be combined with a status icon.
 *
 * ```jsx
 * <now-highlighted-value label="Value" color="warning" show-icon>
 * </now-highlighted-value>
 *
 * <now-highlighted-value label="Tag" color="blue" variant="secondary" icon="clock-outline">
 * </now-highlighted-value>
 * ```
 *
 * @seismicElement now-highlighted-value
 * @summary A colored label with an optional icon to indicate a status.
 * @uib.label Highlighted value
 * @uib.icon highlighted-value
 * @uib.description A colored label with an optional icon to indicate a status.
 * @uib.category primitives
 * @uib.associatedTypes global.core
 * @uib.properties label, variant, color, icon
 * @uib.layout.identifier.children color, icon
 * @uib.layout.identifier.icon image-outline
 *
 */
createEnhancedElement('now-highlighted-value', {
	properties: {
		/**
		 * Text displayed as the highlighted value.
		 * @type {string}
		 * @uib.description Text displayed as the highlighted value.
		 * @uib.defaultValue Heads up
		 * @uib.translatable true
		 */
		label: {required: true, schema: {type: 'string'}},
		/**
		 * Deprecated. Use `color` instead.
		 * @private
		 */
		status: {},
		/**
		 * Sets the highlighted value background color.
		 * @type {('critical'|'high'|'warning'|'moderate'|'positive'|'info'|'low'|'magenta'|'pink'|'orange'|'yellow'|'brown'|'green'|'green-yellow'|'blue'|'gray'|'teal'|'purple')}
		 * @uib.label Color
		 * @uib.description Sets the highlighted value background color.
		 */
		color: {
			default: 'low',
			schema: {
				type: 'string',
				enum: [
					'critical',
					'high',
					'warning',
					'moderate',
					'positive',
					'info',
					'low',
					'magenta',
					'pink',
					'orange',
					'yellow',
					'brown',
					'green',
					'green-yellow',
					'blue',
					'gray',
					'teal',
					'purple'
				]
			}
		},
		/**
		 * Sets the shade of the highlighted value background and the color of
		 * the text and/or icon.
		 * @type {('primary'|'secondary'|'tertiary')}
		 * @uib.label Variant
		 * @uib.description Sets the shade of the highlighted value background and the color of the text and/or icon.
		 */
		variant: {
			default: 'tertiary',
			schema: {type: 'string', enum: ['primary', 'secondary', 'tertiary']}
		},
		/**
		 * Flag that determines whether an icon displays with the label.
		 * @type {boolean}
		 */
		showIcon: {default: false, schema: {type: 'boolean'}},
		/**
		 * When `show-icon` is set, a solid circle displays in the
		 * highlighted value. Set this property with a valid icon name from
		 * the `now-icon` set to display a custom icon instead.
		 * @type {string}
		 * @uib.label Icon
		 * @uib.description Optional icon to display in the highlighted value
		 * @uib.defaultValue presence-fill
		 * @uib.fieldType icon
		 */
		icon: {schema: {type: 'string'}}
	},
	view,
	behaviors: [truncationBehavior, tooltipBehavior],
	styles
});
